"""
DBICKETT_LD48_4-16-2005.py
by Daniel Bickett
dbickett@gmail.com
http://www.heureusement.org/

NOTES:
	- This is by no means the final version.
	- Uploading it to demonstrate the cloud effect, due
	  to the response on the IRC channel.
	- I have removed the recycling mechanisms and replaced
	  them with a more clever way. I don't feel like going
	  through and removing traces of it right now, though,
	  so just ignore them as of this file ;)
"""

import pygame
pygame.init()

import random
import Queue

# Constants
# - Directional
DIR_N = 101
DIR_E = 102
DIR_S = 103
DIR_W = 104

# Centralized Data: 'alloc' dict
# - Will Contain Graphics/Objects
# - Helps differentiate between locals and globals;
#   aids in orientation.

# screen related data
alloc = {}
alloc['screen_res'] = 608, 360
alloc['screen_rect'] = pygame.Rect( ( 0 , 0 ) , alloc['screen_res'] )
alloc['screen'] = pygame.display.set_mode( alloc['screen_res'] )

# load the images
alloc['image'] = {}
alloc['image']['background'] = pygame.image.load("background.gif").convert()
alloc['image']['cloud'] = pygame.image.load("cloud.gif").convert()

for image in alloc['image'].itervalues():
	image.set_colorkey( ( 255 , 140 , 237 ) )

class _Cloud:
	
	def __init__( self , initpos , speed , direction ):
		self.image = alloc['image']['cloud']
		self._setPos( initpos )
		self.speed = speed
		self.direction = direction
		self.recycle = False

	def _setPos( self , pos ):
		self.pos = self.image.get_rect().move( pos )

	def _step( self ):
		# check to see if this cloud needs to be recycled
		if ( self.direction == DIR_E and self.pos.left > alloc['screen_rect'].right ) or \
		   ( self.direction == DIR_W and self.pos.right < 0 ):
			# print "Recycling myself..."
			return False
		else:
			# step in the proper direction at the proper speed
			if self.direction == DIR_E:
				self.pos.right += self.speed
			elif self.direction == DIR_W:
				self.pos.left -= self.speed
			return True

	def _blit( self ):
		alloc['screen'].blit( self.image , self.pos )

	def next( self ):
		if self._step():
			# print "Blitting"
			self._blit()
			return True
		else:
			return False

class Clouds:
	
	def __init__( self , cloud_num = 4 , min_speed = 1 , max_speed = 3 , area = pygame.Rect( 0 , 0 , 608 , 123 ) ):
		# take notes
		self.num = cloud_num
		self.min_speed = min_speed
		self.max_speed = max_speed
		self.area = area
		# clouds' container
		self.clouds = []
		# generate clouds
		for x in range( cloud_num ):
			self.clouds.append( self._genCloud() )
		print self.clouds

	def _genCred( self ):
		# direction
		direction = random.choice( ( DIR_W , DIR_E ) )
		# position
		# FIXME: hard-coded image dimensions
		coord = [0,0]
		if direction == DIR_E:
			coord[0] = -104
		else:
			coord[0] = self.area.right + 104
		coord[1] = random.randint( self.area.top , self.area.bottom - 53 )
		# speed
		speed = random.randint( self.min_speed , self.max_speed )
		# done generating.
		return direction, speed, coord
	
	def _genCloud( self ):
		direction, speed, coord = self._genCred()
		return _Cloud( coord , speed , direction )

	def next( self ):
		for cloud in self.clouds:
			if not cloud.next():
				#print "Recycling cloud..."
				direction, speed, coord = self._genCred()
				cloud._setPos( coord )
				cloud.speed = speed
				cloud.direction = direction

CLOUDS = Clouds()

while True:
	# Blit the background
	alloc['screen'].blit( alloc['image']['background'] , ( 0 , 0 ) )
	# Event Loop
	for event in pygame.event.get():
		if event.type == pygame.QUIT:
			raise SystemExit
	# Cloud Logic
	CLOUDS.next()
	# Update the screen
	pygame.display.update()